package crontab

import (
	"github.com/davecgh/go-spew/spew"
)

// INode 比较的必须继承的接口
type INode interface {
	Compare(v INode) bool

	// GetValue 获取值
	GetValue() interface{}
	// SetValue 设置值
	SetValue(v interface{})

	GetPrev() INode
	SetPrev(INode)
	GetNext() INode
	SetNext(INode)
}

// Node 优先链表的节点
type Node struct {
	INode
	prev, next INode
	// isrelease  bool
	value interface{}
}

// NewNode 创建一个PriorityList 节点
func NewNode(v interface{}) *Node {
	n := new(Node)
	n.SetValue(v)
	return n
}

// GetValue 获取值
func (node *Node) GetValue() interface{} {
	return node.value
}

// SetValue 设置值
func (node *Node) SetValue(v interface{}) {
	node.value = v
}

// GetPrev 获取left值
func (node *Node) GetPrev() INode {
	return node.prev
}

// SetPrev 设置left值
func (node *Node) SetPrev(n INode) {
	node.prev = n
}

// GetNext 设置right值
func (node *Node) GetNext() INode {
	return node.next
}

// SetNext 获取left值
func (node *Node) SetNext(n INode) {
	node.next = n
}

// PriorityList 优先列表
type PriorityList struct {
	head INode
}

func (pl *PriorityList) String() string {
	content := "["

	cur := pl.head
	for cur != nil {
		content += spew.Sprint(cur.GetValue()) + "<->"
		cur = cur.GetNext()
	}

	if content[len(content)-3:] == "<->" {
		content = content[:len(content)-3]
	}

	content += "]"
	return content
}

func (pl *PriorityList) InsertValue(node INode) {

	if pl.head == nil {
		pl.head = node
	} else {
		cur := pl.head
		var prev INode
		for cur != nil {
			if !cur.Compare(node) {
				// 插入该值
				prev = cur.GetPrev()
				if prev == nil {
					node.SetNext(pl.head)
					pl.head = node
				} else {
					prev.SetNext(node)
					node.SetNext(cur)
					cur.SetPrev(node)
					node.SetPrev(prev)
					return
				}
			}
			cur = cur.GetNext()
		}
		prev.SetNext(node)
	}
}

// New 创建 PriorityList
func New() *PriorityList {
	return new(PriorityList)
}
